@@ -4,69 +4,94 @@
 //! macro-expanded files, but we need to present them to the users in terms of
 //! original files. So we need to map the ranges.
 
-mod fixes;
-mod field_shorthand;
+mod break_outside_of_loop;
+mod inactive_code;
+mod incorrect_case;
+mod macro_error;
+mod mismatched_arg_count;
+mod missing_fields;
+mod missing_match_arms;
+mod missing_ok_or_some_in_tail_expr;
+mod missing_unsafe;
+mod no_such_field;
+mod remove_this_semicolon;
+mod replace_filter_map_next_with_find_map;
+mod unimplemented_builtin_macro;
 mod unlinked_file;
+mod unresolved_extern_crate;
+mod unresolved_import;
+mod unresolved_macro_call;
+mod unresolved_module;
+mod unresolved_proc_macro;
 
-use std::cell::RefCell;
+mod field_shorthand;
 
-use hir::{
-    db::AstDatabase,
-    diagnostics::{Diagnostic as _, DiagnosticCode, DiagnosticSinkBuilder},
-    InFile, Semantics,
-};
+use hir::{diagnostics::AnyDiagnostic, Semantics};
 use ide_assists::AssistResolveStrategy;
 use ide_db::{base_db::SourceDatabase, RootDatabase};
 use itertools::Itertools;
 use rustc_hash::FxHashSet;
 use syntax::{
     ast::{self, AstNode},
-    SyntaxNode, SyntaxNodePtr, TextRange, TextSize,
+    SyntaxNode, TextRange,
 };
 use text_edit::TextEdit;
 use unlinked_file::UnlinkedFile;
 
 use crate::{Assist, AssistId, AssistKind, FileId, Label, SourceChange};
 
-use self::fixes::DiagnosticWithFixes;
+#[derive(Copy, Clone, Debug, PartialEq)]
+pub struct DiagnosticCode(pub &'static str);
+
+impl DiagnosticCode {
+    pub fn as_str(&self) -> &str {
+        self.0
+    }
+}
 
 #[derive(Debug)]
 pub struct Diagnostic {
-    // pub name: Option<String>,
+    pub code: DiagnosticCode,
     pub message: String,
     pub range: TextRange,
     pub severity: Severity,
-    pub fixes: Option<Vec<Assist>>,
     pub unused: bool,
-    pub code: Option<DiagnosticCode>,
+    pub experimental: bool,
+    pub fixes: Option<Vec<Assist>>,
 }
 
 impl Diagnostic {
-    fn error(range: TextRange, message: String) -> Self {
-        Self { message, range, severity: Severity::Error, fixes: None, unused: false, code: None }
-    }
-
-    fn hint(range: TextRange, message: String) -> Self {
-        Self {
+    fn new(code: &'static str, message: impl Into<String>, range: TextRange) -> Diagnostic {
+        let message = message.into();
+        Diagnostic {
+            code: DiagnosticCode(code),
             message,
             range,
-            severity: Severity::WeakWarning,
-            fixes: None,
+            severity: Severity::Error,
             unused: false,
-            code: None,
+            experimental: false,
+            fixes: None,
         }
     }
 
-    fn with_fixes(self, fixes: Option<Vec<Assist>>) -> Self {
-        Self { fixes, ..self }
+    fn experimental(mut self) -> Diagnostic {
+        self.experimental = true;
+        self
     }
 
-    fn with_unused(self, unused: bool) -> Self {
-        Self { unused, ..self }
+    fn severity(mut self, severity: Severity) -> Diagnostic {
+        self.severity = severity;
+        self
     }
 
-    fn with_code(self, code: Option<DiagnosticCode>) -> Self {
-        Self { code, ..self }
+    fn with_fixes(mut self, fixes: Option<Vec<Assist>>) -> Diagnostic {
+        self.fixes = fixes;
+        self
+    }
+
+    fn with_unused(mut self, unused: bool) -> Diagnostic {
+        self.unused = unused;
+        self
     }
 }
 
@@ -82,6 +107,12 @@ pub struct DiagnosticsConfig {
     pub disabled: FxHashSet<String>,
 }
 
+struct DiagnosticsContext<'a> {
+    config: &'a DiagnosticsConfig,
+    sema: Semantics<'a, RootDatabase>,
+    resolve: &'a AssistResolveStrategy,
+}
+
 pub(crate) fn diagnostics(
     db: &RootDatabase,
     config: &DiagnosticsConfig,
@@ -95,139 +126,64 @@ pub(crate) fn diagnostics(
 
     // [#34344] Only take first 128 errors to prevent slowing down editor/ide, the number 128 is chosen arbitrarily.
     res.extend(
-        parse
-            .errors()
-            .iter()
-            .take(128)
-            .map(|err| Diagnostic::error(err.range(), format!("Syntax Error: {}", err))),
+        parse.errors().iter().take(128).map(|err| {
+            Diagnostic::new("syntax-error", format!("Syntax Error: {}", err), err.range())
+        }),
     );
 
     for node in parse.tree().syntax().descendants() {
         check_unnecessary_braces_in_use_statement(&mut res, file_id, &node);
         field_shorthand::check(&mut res, file_id, &node);
     }
-    let res = RefCell::new(res);
-    let sink_builder = DiagnosticSinkBuilder::new()
-        .on::<hir::diagnostics::UnresolvedModule, _>(|d| {
-            res.borrow_mut().push(diagnostic_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::MissingFields, _>(|d| {
-            res.borrow_mut().push(diagnostic_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::MissingOkOrSomeInTailExpr, _>(|d| {
-            res.borrow_mut().push(diagnostic_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::NoSuchField, _>(|d| {
-            res.borrow_mut().push(diagnostic_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::RemoveThisSemicolon, _>(|d| {
-            res.borrow_mut().push(diagnostic_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::IncorrectCase, _>(|d| {
-            res.borrow_mut().push(warning_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::ReplaceFilterMapNextWithFindMap, _>(|d| {
-            res.borrow_mut().push(warning_with_fix(d, &sema, resolve));
-        })
-        .on::<hir::diagnostics::InactiveCode, _>(|d| {
-            // If there's inactive code somewhere in a macro, don't propagate to the call-site.
-            if d.display_source().file_id.expansion_info(db).is_some() {
-                return;
-            }
 
-            // Override severity and mark as unused.
-            res.borrow_mut().push(
-                Diagnostic::hint(
-                    sema.diagnostics_display_range(d.display_source()).range,
-                    d.message(),
-                )
-                .with_unused(true)
-                .with_code(Some(d.code())),
-            );
-        })
-        .on::<UnlinkedFile, _>(|d| {
-            // Limit diagnostic to the first few characters in the file. This matches how VS Code
-            // renders it with the full span, but on other editors, and is less invasive.
-            let range = sema.diagnostics_display_range(d.display_source()).range;
-            let range = range.intersect(TextRange::up_to(TextSize::of("..."))).unwrap_or(range);
-
-            // Override severity and mark as unused.
-            res.borrow_mut().push(
-                Diagnostic::hint(range, d.message())
-                    .with_fixes(d.fixes(&sema, resolve))
-                    .with_code(Some(d.code())),
-            );
-        })
-        .on::<hir::diagnostics::UnresolvedProcMacro, _>(|d| {
-            // Use more accurate position if available.
-            let display_range = d
-                .precise_location
-                .unwrap_or_else(|| sema.diagnostics_display_range(d.display_source()).range);
-
-            // FIXME: it would be nice to tell the user whether proc macros are currently disabled
-            res.borrow_mut()
-                .push(Diagnostic::hint(display_range, d.message()).with_code(Some(d.code())));
-        })
-        .on::<hir::diagnostics::UnresolvedMacroCall, _>(|d| {
-            let last_path_segment = sema.db.parse_or_expand(d.file).and_then(|root| {
-                d.node
-                    .to_node(&root)
-                    .path()
-                    .and_then(|it| it.segment())
-                    .and_then(|it| it.name_ref())
-                    .map(|it| InFile::new(d.file, SyntaxNodePtr::new(it.syntax())))
-            });
-            let diagnostics = last_path_segment.unwrap_or_else(|| d.display_source());
-            let display_range = sema.diagnostics_display_range(diagnostics).range;
-            res.borrow_mut()
-                .push(Diagnostic::error(display_range, d.message()).with_code(Some(d.code())));
-        })
-        // Only collect experimental diagnostics when they're enabled.
-        .filter(|diag| !(diag.is_experimental() && config.disable_experimental))
-        .filter(|diag| !config.disabled.contains(diag.code().as_str()));
-
-    // Finalize the `DiagnosticSink` building process.
-    let mut sink = sink_builder
-        // Diagnostics not handled above get no fix and default treatment.
-        .build(|d| {
-            res.borrow_mut().push(
-                Diagnostic::error(
-                    sema.diagnostics_display_range(d.display_source()).range,
-                    d.message(),
-                )
-                .with_code(Some(d.code())),
-            );
-        });
-
-    match sema.to_module_def(file_id) {
-        Some(m) => m.diagnostics(db, &mut sink),
-        None => {
-            sink.push(UnlinkedFile { file_id, node: SyntaxNodePtr::new(&parse.tree().syntax()) });
-        }
+    let mut diags = Vec::new();
+    let module = sema.to_module_def(file_id);
+    if let Some(m) = module {
+        m.diagnostics(db, &mut diags)
     }
 
-    drop(sink);
-    res.into_inner()
-}
+    let ctx = DiagnosticsContext { config, sema, resolve };
+    if module.is_none() {
+        let d = UnlinkedFile { file: file_id };
+        let d = unlinked_file::unlinked_file(&ctx, &d);
+        res.push(d)
+    }
 
-fn diagnostic_with_fix<D: DiagnosticWithFixes>(
-    d: &D,
-    sema: &Semantics<RootDatabase>,
-    resolve: &AssistResolveStrategy,
-) -> Diagnostic {
-    Diagnostic::error(sema.diagnostics_display_range(d.display_source()).range, d.message())
-        .with_fixes(d.fixes(&sema, resolve))
-        .with_code(Some(d.code()))
-}
+    for diag in diags {
+        #[rustfmt::skip]
+        let d = match diag {
+            AnyDiagnostic::BreakOutsideOfLoop(d) => break_outside_of_loop::break_outside_of_loop(&ctx, &d),
+            AnyDiagnostic::IncorrectCase(d) => incorrect_case::incorrect_case(&ctx, &d),
+            AnyDiagnostic::MacroError(d) => macro_error::macro_error(&ctx, &d),
+            AnyDiagnostic::MismatchedArgCount(d) => mismatched_arg_count::mismatched_arg_count(&ctx, &d),
+            AnyDiagnostic::MissingFields(d) => missing_fields::missing_fields(&ctx, &d),
+            AnyDiagnostic::MissingMatchArms(d) => missing_match_arms::missing_match_arms(&ctx, &d),
+            AnyDiagnostic::MissingOkOrSomeInTailExpr(d) => missing_ok_or_some_in_tail_expr::missing_ok_or_some_in_tail_expr(&ctx, &d),
+            AnyDiagnostic::MissingUnsafe(d) => missing_unsafe::missing_unsafe(&ctx, &d),
+            AnyDiagnostic::NoSuchField(d) => no_such_field::no_such_field(&ctx, &d),
+            AnyDiagnostic::RemoveThisSemicolon(d) => remove_this_semicolon::remove_this_semicolon(&ctx, &d),
+            AnyDiagnostic::ReplaceFilterMapNextWithFindMap(d) => replace_filter_map_next_with_find_map::replace_filter_map_next_with_find_map(&ctx, &d),
+            AnyDiagnostic::UnimplementedBuiltinMacro(d) => unimplemented_builtin_macro::unimplemented_builtin_macro(&ctx, &d),
+            AnyDiagnostic::UnresolvedExternCrate(d) => unresolved_extern_crate::unresolved_extern_crate(&ctx, &d),
+            AnyDiagnostic::UnresolvedImport(d) => unresolved_import::unresolved_import(&ctx, &d),
+            AnyDiagnostic::UnresolvedMacroCall(d) => unresolved_macro_call::unresolved_macro_call(&ctx, &d),
+            AnyDiagnostic::UnresolvedModule(d) => unresolved_module::unresolved_module(&ctx, &d),
+            AnyDiagnostic::UnresolvedProcMacro(d) => unresolved_proc_macro::unresolved_proc_macro(&ctx, &d),
 
-fn warning_with_fix<D: DiagnosticWithFixes>(
-    d: &D,
-    sema: &Semantics<RootDatabase>,
-    resolve: &AssistResolveStrategy,
-) -> Diagnostic {
-    Diagnostic::hint(sema.diagnostics_display_range(d.display_source()).range, d.message())
-        .with_fixes(d.fixes(&sema, resolve))
-        .with_code(Some(d.code()))
+            AnyDiagnostic::InactiveCode(d) => match inactive_code::inactive_code(&ctx, &d) {
+                Some(it) => it,
+                None => continue,
+            }
+        };
+        res.push(d)
+    }
+
+    res.retain(|d| {
+        !ctx.config.disabled.contains(d.code.as_str())
+            && !(ctx.config.disable_experimental && d.experimental)
+    });
+
+    res
 }
 
 fn check_unnecessary_braces_in_use_statement(
@@ -255,13 +211,18 @@ fn check_unnecessary_braces_in_use_statement(
                 });
 
         acc.push(
-            Diagnostic::hint(use_range, "Unnecessary braces in use statement".to_string())
-                .with_fixes(Some(vec![fix(
-                    "remove_braces",
-                    "Remove unnecessary braces",
-                    SourceChange::from_text_edit(file_id, edit),
-                    use_range,
-                )])),
+            Diagnostic::new(
+                "unnecessary-braces",
+                "Unnecessary braces in use statement".to_string(),
+                use_range,
+            )
+            .severity(Severity::WeakWarning)
+            .with_fixes(Some(vec![fix(
+                "remove_braces",
+                "Remove unnecessary braces",
+                SourceChange::from_text_edit(file_id, edit),
+                use_range,
+            )])),
         );
     }
 
@@ -339,8 +300,8 @@ mod tests {
             )
             .unwrap()
             .pop()
-            .unwrap();
-        let fix = &diagnostic.fixes.unwrap()[nth];
+            .expect("no diagnostics");
+        let fix = &diagnostic.fixes.expect("diagnostic misses fixes")[nth];
         let actual = {
             let source_change = fix.source_change.as_ref().unwrap();
             let file_id = *source_change.source_file_edits.keys().next().unwrap();
@@ -360,8 +321,9 @@ mod tests {
             file_position.offset
         );
     }
+
     /// Checks that there's a diagnostic *without* fix at `$0`.
-    fn check_no_fix(ra_fixture: &str) {
+    pub(crate) fn check_no_fix(ra_fixture: &str) {
         let (analysis, file_position) = fixture::position(ra_fixture);
         let diagnostic = analysis
             .diagnostics(
@@ -375,21 +337,6 @@ mod tests {
         assert!(diagnostic.fixes.is_none(), "got a fix when none was expected: {:?}", diagnostic);
     }
 
-    /// Takes a multi-file input fixture with annotated cursor position and checks that no diagnostics
-    /// apply to the file containing the cursor.
-    pub(crate) fn check_no_diagnostics(ra_fixture: &str) {
-        let (analysis, files) = fixture::files(ra_fixture);
-        let diagnostics = files
-            .into_iter()
-            .flat_map(|file_id| {
-                analysis
-                    .diagnostics(&DiagnosticsConfig::default(), AssistResolveStrategy::All, file_id)
-                    .unwrap()
-            })
-            .collect::<Vec<_>>();
-        assert_eq!(diagnostics.len(), 0, "unexpected diagnostics:\n{:#?}", diagnostics);
-    }
-
     pub(crate) fn check_expect(ra_fixture: &str, expect: Expect) {
         let (analysis, file_id) = fixture::file(ra_fixture);
         let diagnostics = analysis
@@ -398,90 +345,31 @@ mod tests {
         expect.assert_debug_eq(&diagnostics)
     }
 
+    #[track_caller]
     pub(crate) fn check_diagnostics(ra_fixture: &str) {
-        let (analysis, file_id) = fixture::file(ra_fixture);
-        let diagnostics = analysis
-            .diagnostics(&DiagnosticsConfig::default(), AssistResolveStrategy::All, file_id)
-            .unwrap();
-
-        let expected = extract_annotations(&*analysis.file_text(file_id).unwrap());
-        let actual = diagnostics.into_iter().map(|d| (d.range, d.message)).collect::<Vec<_>>();
-        assert_eq!(expected, actual);
+        let mut config = DiagnosticsConfig::default();
+        config.disabled.insert("inactive-code".to_string());
+        check_diagnostics_with_config(config, ra_fixture)
     }
 
-    #[test]
-    fn test_unresolved_macro_range() {
-        check_diagnostics(
-            r#"
-foo::bar!(92);
-   //^^^ unresolved macro `foo::bar!`
-"#,
-        );
-    }
+    #[track_caller]
+    pub(crate) fn check_diagnostics_with_config(config: DiagnosticsConfig, ra_fixture: &str) {
+        let (analysis, files) = fixture::files(ra_fixture);
+        for file_id in files {
+            let diagnostics =
+                analysis.diagnostics(&config, AssistResolveStrategy::All, file_id).unwrap();
 
-    #[test]
-    fn unresolved_import_in_use_tree() {
-        // Only the relevant part of a nested `use` item should be highlighted.
-        check_diagnostics(
-            r#"
-use does_exist::{Exists, DoesntExist};
-                       //^^^^^^^^^^^ unresolved import
-
-use {does_not_exist::*, does_exist};
-   //^^^^^^^^^^^^^^^^^ unresolved import
-
-use does_not_exist::{
-    a,
-  //^ unresolved import
-    b,
-  //^ unresolved import
-    c,
-  //^ unresolved import
-};
-
-mod does_exist {
-    pub struct Exists;
-}
-"#,
-        );
-    }
-
-    #[test]
-    fn range_mapping_out_of_macros() {
-        // FIXME: this is very wrong, but somewhat tricky to fix.
-        check_fix(
-            r#"
-fn some() {}
-fn items() {}
-fn here() {}
-
-macro_rules! id { ($($tt:tt)*) => { $($tt)*}; }
-
-fn main() {
-    let _x = id![Foo { a: $042 }];
-}
-
-pub struct Foo { pub a: i32, pub b: i32 }
-"#,
-            r#"
-fn some(, b: () ) {}
-fn items() {}
-fn here() {}
-
-macro_rules! id { ($($tt:tt)*) => { $($tt)*}; }
-
-fn main() {
-    let _x = id![Foo { a: 42 }];
-}
-
-pub struct Foo { pub a: i32, pub b: i32 }
-"#,
-        );
+            let expected = extract_annotations(&*analysis.file_text(file_id).unwrap());
+            let mut actual =
+                diagnostics.into_iter().map(|d| (d.range, d.message)).collect::<Vec<_>>();
+            actual.sort_by_key(|(range, _)| range.start());
+            assert_eq!(expected, actual);
+        }
     }
 
     #[test]
     fn test_check_unnecessary_braces_in_use_statement() {
-        check_no_diagnostics(
+        check_diagnostics(
             r#"
 use a;
 use a::{c, d::e};
@@ -494,7 +382,7 @@ mod a {
 }
 "#,
         );
-        check_no_diagnostics(
+        check_diagnostics(
             r#"
 use a;
 use a::{
@@ -580,138 +468,31 @@ mod a {
     }
 
     #[test]
-    fn unlinked_file_prepend_first_item() {
-        cov_mark::check!(unlinked_file_prepend_before_first_item);
-        // Only tests the first one for `pub mod` since the rest are the same
-        check_fixes(
+    fn import_extern_crate_clash_with_inner_item() {
+        // This is more of a resolver test, but doesn't really work with the hir_def testsuite.
+
+        check_diagnostics(
             r#"
-//- /main.rs
-fn f() {}
-//- /foo.rs
-$0
-"#,
-            vec![
-                r#"
-mod foo;
+//- /lib.rs crate:lib deps:jwt
+mod permissions;
 
-fn f() {}
-"#,
-                r#"
-pub mod foo;
+use permissions::jwt;
 
-fn f() {}
-"#,
-            ],
-        );
-    }
+fn f() {
+    fn inner() {}
+    jwt::Claims {}; // should resolve to the local one with 0 fields, and not get a diagnostic
+}
 
-    #[test]
-    fn unlinked_file_append_mod() {
-        cov_mark::check!(unlinked_file_append_to_existing_mods);
-        check_fix(
-            r#"
-//- /main.rs
-//! Comment on top
+//- /permissions.rs
+pub mod jwt  {
+    pub struct Claims {}
+}
 
-mod preexisting;
-
-mod preexisting2;
-
-struct S;
-
-mod preexisting_bottom;)
-//- /foo.rs
-$0
-"#,
-            r#"
-//! Comment on top
-
-mod preexisting;
-
-mod preexisting2;
-mod foo;
-
-struct S;
-
-mod preexisting_bottom;)
-"#,
-        );
-    }
-
-    #[test]
-    fn unlinked_file_insert_in_empty_file() {
-        cov_mark::check!(unlinked_file_empty_file);
-        check_fix(
-            r#"
-//- /main.rs
-//- /foo.rs
-$0
-"#,
-            r#"
-mod foo;
-"#,
-        );
-    }
-
-    #[test]
-    fn unlinked_file_old_style_modrs() {
-        check_fix(
-            r#"
-//- /main.rs
-mod submod;
-//- /submod/mod.rs
-// in mod.rs
-//- /submod/foo.rs
-$0
-"#,
-            r#"
-// in mod.rs
-mod foo;
-"#,
-        );
-    }
-
-    #[test]
-    fn unlinked_file_new_style_mod() {
-        check_fix(
-            r#"
-//- /main.rs
-mod submod;
-//- /submod.rs
-//- /submod/foo.rs
-$0
-"#,
-            r#"
-mod foo;
-"#,
-        );
-    }
-
-    #[test]
-    fn unlinked_file_with_cfg_off() {
-        cov_mark::check!(unlinked_file_skip_fix_when_mod_already_exists);
-        check_no_fix(
-            r#"
-//- /main.rs
-#[cfg(never)]
-mod foo;
-
-//- /foo.rs
-$0
-"#,
-        );
-    }
-
-    #[test]
-    fn unlinked_file_with_cfg_on() {
-        check_no_diagnostics(
-            r#"
-//- /main.rs
-#[cfg(not(never))]
-mod foo;
-
-//- /foo.rs
-"#,
+//- /jwt/lib.rs crate:jwt
+pub struct Claims {
+    field: u8,
+}
+        "#,
         );
     }
 }
